import spidev
import time
import RPi.GPIO as GPIO
import binascii

# RX buffer size
ENC28J60_ETH_RX_BUFFER_SIZE     = (1518)
ENC28J60_ETH_TX_BUFFER_SIZE     = (1518)
# TX error codes
ENC28J60_ETH_TX_ERR_MSGSIZE	    = (-1)
ENC28J60_ETH_TX_ERR_LINKDOWN	= (-2)
ENC28J60_ETH_TX_ERR_SEND	    = (-3)

# SPI command set
ENC28J60_CMD_RCR		        = 0x00
ENC28J60_CMD_RBM		        = 0x3A
ENC28J60_CMD_WCR		        = 0x40
ENC28J60_CMD_WBM	        	= 0x7A
ENC28J60_CMD_BFS	        	= 0x80
ENC28J60_CMD_BFC        		= 0xA0
ENC28J60_CMD_SRC		        = 0xFF
# ENC28J60 banks
BANK_0				            = 0x0000
BANK_1				            = 0x0100
BANK_2				            = 0x0200
BANK_3				            = 0x0300
# ENC28J60 register types
ETH_REG_TYPE                    = 0x0000
MAC_REG_TYPE                    = 0x1000
MII_REG_TYPE                    = 0x2000
PHY_REG_TYPE                    = 0x3000
# Related masks
REG_TYPE_MASK                   = (0xF000)
REG_BANK_MASK                   = (0x0F00)
REG_ADDR_MASK                   = (0x001F)

# ENC28J60 registers
ENC28J60_ERDPTL			= (ETH_REG_TYPE | BANK_0 | 0x00)
ENC28J60_ERDPTH			= (ETH_REG_TYPE | BANK_0 | 0x01)
ENC28J60_EWRPTL			= (ETH_REG_TYPE | BANK_0 | 0x02)
ENC28J60_EWRPTH			= (ETH_REG_TYPE | BANK_0 | 0x03)
ENC28J60_ETXSTL			= (ETH_REG_TYPE | BANK_0 | 0x04)
ENC28J60_ETXSTH			= (ETH_REG_TYPE | BANK_0 | 0x05)
ENC28J60_ETXNDL			= (ETH_REG_TYPE | BANK_0 | 0x06)
ENC28J60_ETXNDH			= (ETH_REG_TYPE | BANK_0 | 0x07)
ENC28J60_ERXSTL			= (ETH_REG_TYPE | BANK_0 | 0x08)
ENC28J60_ERXSTH			= (ETH_REG_TYPE | BANK_0 | 0x09)
ENC28J60_ERXNDL			= (ETH_REG_TYPE | BANK_0 | 0x0A)
ENC28J60_ERXNDH			= (ETH_REG_TYPE | BANK_0 | 0x0B)
ENC28J60_ERXRDPTL		= (ETH_REG_TYPE | BANK_0 | 0x0C)
ENC28J60_ERXRDPTH		= (ETH_REG_TYPE | BANK_0 | 0x0D)
ENC28J60_ERXWRPTL		= (ETH_REG_TYPE | BANK_0 | 0x0E)
ENC28J60_ERXWRPTH		= (ETH_REG_TYPE | BANK_0 | 0x0F)
ENC28J60_EDMASTL		= (ETH_REG_TYPE | BANK_0 | 0x10)
ENC28J60_EDMASTH		= (ETH_REG_TYPE | BANK_0 | 0x11)
ENC28J60_EDMANDL		= (ETH_REG_TYPE | BANK_0 | 0x12)
ENC28J60_EDMANDH		= (ETH_REG_TYPE | BANK_0 | 0x13)
ENC28J60_EDMADSTL		= (ETH_REG_TYPE | BANK_0 | 0x14)
ENC28J60_EDMADSTH		= (ETH_REG_TYPE | BANK_0 | 0x15)
ENC28J60_EDMACSL		= (ETH_REG_TYPE | BANK_0 | 0x16)
ENC28J60_EDMACSH		= (ETH_REG_TYPE | BANK_0 | 0x17)
ENC28J60_EIE   			= (ETH_REG_TYPE | BANK_0 | 0x1B)
ENC28J60_EIR   			= (ETH_REG_TYPE | BANK_0 | 0x1C)
ENC28J60_ESTAT 			= (ETH_REG_TYPE | BANK_0 | 0x1D)
ENC28J60_ECON2 			= (ETH_REG_TYPE | BANK_0 | 0x1E)
ENC28J60_ECON1 			= (ETH_REG_TYPE | BANK_0 | 0x1F)
ENC28J60_EHT0  			= (ETH_REG_TYPE | BANK_1 | 0x00)
ENC28J60_EHT1  			= (ETH_REG_TYPE | BANK_1 | 0x01)
ENC28J60_EHT2  			= (ETH_REG_TYPE | BANK_1 | 0x02)
ENC28J60_EHT3  			= (ETH_REG_TYPE | BANK_1 | 0x03)
ENC28J60_EHT4  			= (ETH_REG_TYPE | BANK_1 | 0x04)
ENC28J60_EHT5  			= (ETH_REG_TYPE | BANK_1 | 0x05)
ENC28J60_EHT6  			= (ETH_REG_TYPE | BANK_1 | 0x06)
ENC28J60_EHT7  			= (ETH_REG_TYPE | BANK_1 | 0x07)
ENC28J60_EPMM0 			= (ETH_REG_TYPE | BANK_1 | 0x08)
ENC28J60_EPMM1 			= (ETH_REG_TYPE | BANK_1 | 0x09)
ENC28J60_EPMM2 			= (ETH_REG_TYPE | BANK_1 | 0x0A)
ENC28J60_EPMM3 			= (ETH_REG_TYPE | BANK_1 | 0x0B)
ENC28J60_EPMM4 			= (ETH_REG_TYPE | BANK_1 | 0x0C)
ENC28J60_EPMM5 			= (ETH_REG_TYPE | BANK_1 | 0x0D)
ENC28J60_EPMM6 			= (ETH_REG_TYPE | BANK_1 | 0x0E)
ENC28J60_EPMM7 			= (ETH_REG_TYPE | BANK_1 | 0x0F)
ENC28J60_EPMCSL			= (ETH_REG_TYPE | BANK_1 | 0x10)
ENC28J60_EPMCSH			= (ETH_REG_TYPE | BANK_1 | 0x11)
ENC28J60_EPMOL 			= (ETH_REG_TYPE | BANK_1 | 0x14)
ENC28J60_EPMOH 			= (ETH_REG_TYPE | BANK_1 | 0x15)
ENC28J60_EWOLIE			= (ETH_REG_TYPE | BANK_1 | 0x16)
ENC28J60_EWOLIR			= (ETH_REG_TYPE | BANK_1 | 0x17)
ENC28J60_ERXFCON		= (ETH_REG_TYPE | BANK_1 | 0x18)
ENC28J60_EPKTCNT		= (ETH_REG_TYPE | BANK_1 | 0x19)
ENC28J60_MACON1			= (MAC_REG_TYPE | BANK_2 | 0x00)
ENC28J60_MACON2			= (MAC_REG_TYPE | BANK_2 | 0x01)
ENC28J60_MACON3			= (MAC_REG_TYPE | BANK_2 | 0x02)
ENC28J60_MACON4			= (MAC_REG_TYPE | BANK_2 | 0x03)
ENC28J60_MABBIPG		= (MAC_REG_TYPE | BANK_2 | 0x04)
ENC28J60_MAIPGL			= (MAC_REG_TYPE | BANK_2 | 0x06)
ENC28J60_MAIPGH			= (MAC_REG_TYPE | BANK_2 | 0x07)
ENC28J60_MACLCON1		= (MAC_REG_TYPE | BANK_2 | 0x08)
ENC28J60_MACLCON2		= (MAC_REG_TYPE | BANK_2 | 0x09)
ENC28J60_MAMXFLL		= (MAC_REG_TYPE | BANK_2 | 0x0A)
ENC28J60_MAMXFLH		= (MAC_REG_TYPE | BANK_2 | 0x0B)
ENC28J60_MAPHSUP		= (MAC_REG_TYPE | BANK_2 | 0x0D)
ENC28J60_MICON 			= (MII_REG_TYPE | BANK_2 | 0x11)
ENC28J60_MICMD 			= (MII_REG_TYPE | BANK_2 | 0x12)
ENC28J60_MIREGADR		= (MII_REG_TYPE | BANK_2 | 0x14)
ENC28J60_MIWRL 			= (MII_REG_TYPE | BANK_2 | 0x16)
ENC28J60_MIWRH 			= (MII_REG_TYPE | BANK_2 | 0x17)
ENC28J60_MIRDL 			= (MII_REG_TYPE | BANK_2 | 0x18)
ENC28J60_MIRDH 			= (MII_REG_TYPE | BANK_2 | 0x19)
ENC28J60_MAADR1			= (MAC_REG_TYPE | BANK_3 | 0x00)
ENC28J60_MAADR0			= (MAC_REG_TYPE | BANK_3 | 0x01)
ENC28J60_MAADR3			= (MAC_REG_TYPE | BANK_3 | 0x02)
ENC28J60_MAADR2			= (MAC_REG_TYPE | BANK_3 | 0x03)
ENC28J60_MAADR5			= (MAC_REG_TYPE | BANK_3 | 0x04)
ENC28J60_MAADR4			= (MAC_REG_TYPE | BANK_3 | 0x05)
ENC28J60_EBSTSD			= (ETH_REG_TYPE | BANK_3 | 0x06)
ENC28J60_EBSTCON		= (ETH_REG_TYPE | BANK_3 | 0x07)
ENC28J60_EBSTCSL		= (ETH_REG_TYPE | BANK_3 | 0x08)
ENC28J60_EBSTCSH		= (ETH_REG_TYPE | BANK_3 | 0x09)
ENC28J60_MISTAT			= (MII_REG_TYPE | BANK_3 | 0x0A)
ENC28J60_EREVID			= (ETH_REG_TYPE | BANK_3 | 0x12)
ENC28J60_ECOCON			= (ETH_REG_TYPE | BANK_3 | 0x15)
ENC28J60_EFLOCON		= (ETH_REG_TYPE | BANK_3 | 0x17)
ENC28J60_EPAUSL			= (ETH_REG_TYPE | BANK_3 | 0x18)
ENC28J60_EPAUSH			= (ETH_REG_TYPE | BANK_3 | 0x19)
# Receive and transmit buffers
ENC28J60_RX_BUFFER_START	= 0x0000
ENC28J60_RX_BUFFER_STOP		= 0x17FF
ENC28J60_TX_BUFFER_START	= 0x1800
ENC28J60_TX_BUFFER_STOP		= 0x1FFF

# ENC28J60 PHY registers
ENC28J60_PHCON1			= (PHY_REG_TYPE | 0x00)
ENC28J60_PHSTAT1        = (PHY_REG_TYPE | 0x01)
ENC28J60_PHID1 			= (PHY_REG_TYPE | 0x02)
ENC28J60_PHID2 			= (PHY_REG_TYPE | 0x03)
ENC28J60_PHCON2			= (PHY_REG_TYPE | 0x10)
ENC28J60_PHSTAT2        = (PHY_REG_TYPE | 0x11)
ENC28J60_PHIE  			= (PHY_REG_TYPE | 0x12)
ENC28J60_PHIR  			= (PHY_REG_TYPE | 0x13)
ENC28J60_PHLCON			= (PHY_REG_TYPE | 0x14)

# Ethernet Interrupt Enable register
EIE_INTIE                   = (0x80)
EIE_PKTIE                   = (0x40)
EIE_DMAIE                   = (0x20)
EIE_LINKIE                  = (0x10)
EIE_TXIE                    = (0x08)
EIE_WOLIE                   = (0x04)
EIE_TXERIE                  = (0x02)
EIE_RXERIE                  = (0x01)

# Ethernet Interrupt Request register
EIR_PKTIF                   = (0x40)
EIR_DMAIF                   = (0x20)
EIR_LINKIF                  = (0x10)
EIR_TXIF                    = (0x08)
EIR_WOLIF                   = (0x04)
EIR_TXERIF                  = (0x02)
EIR_RXERIF                  = (0x01)

# Ethernet Status register
ESTAT_INT                   = (0x80)
ESTAT_R6                    = (0x40)
ESTAT_R5                    = (0x20)
ESTAT_LATECOL               = (0x10)
ESTAT_RXBUSY                = (0x04)
ESTAT_TXABRT                = (0x02)
ESTAT_CLKRDY                = (0x01)

# Ethernet Control 2 register
ECON2_AUTOINC               = (0x80)
ECON2_PKTDEC                = (0x40)
ECON2_PWRSV                 = (0x20)
ECON2_VRPS                  = (0x08)

# Ethernet Control 1 register
ECON1_TXRST                 = (0x80)
ECON1_RXRST                 = (0x40)
ECON1_DMAST                 = (0x20)
ECON1_CSUMEN                = (0x10)
ECON1_TXRTS                 = (0x08)
ECON1_RXEN                  = (0x04)
ECON1_BSEL1                 = (0x02)
ECON1_BSEL0                 = (0x01)

# Ethernet Wake-Up On LAN Interrupt Enable register
EWOLIE_UCWOLIE              = (0x80)
EWOLIE_AWOLIE               = (0x40)
EWOLIE_PMWOLIE              = (0x10)
EWOLIE_MPWOLIE              = (0x08)
EWOLIE_HTWOLIE              = (0x04)
EWOLIE_MCWOLIE              = (0x02)
EWOLIE_BCWOLIE              = (0x01)

# Ethernet Wake-Up On LAN Interrupt Request register
EWOLIR_UCWOLIF              = (0x80)
EWOLIR_AWOLIF               = (0x40)
EWOLIR_PMWOLIF              = (0x10)
EWOLIR_MPWOLIF              = (0x08)
EWOLIR_HTWOLIF              = (0x04)
EWOLIR_MCWOLIF              = (0x02)
EWOLIR_BCWOLIF              = (0x01)

# Receive Filter Control register
ERXFCON_UCEN                = (0x80)
ERXFCON_ANDOR               = (0x40)
ERXFCON_CRCEN               = (0x20)
ERXFCON_PMEN                = (0x10)
ERXFCON_MPEN                = (0x08)
ERXFCON_HTEN                = (0x04)
ERXFCON_MCEN                = (0x02)
ERXFCON_BCEN                = (0x01)

# MAC Control 1 register
MACON1_LOOPBK               = (0x10)
MACON1_TXPAUS               = (0x08)
MACON1_RXPAUS               = (0x04)
MACON1_PASSALL              = (0x02)
MACON1_MARXEN               = (0x01)

# MAC Control 2 register
MACON2_MARST                = (0x80)
MACON2_RNDRST               = (0x40)
MACON2_MARXRST              = (0x08)
MACON2_RFUNRST              = (0x04)
MACON2_MATXRST              = (0x02)
MACON2_TFUNRST              = (0x01)

# MAC Control 3 register
MACON3_PADCFG               = (0xE0)
MACON3_PADCFG_NO            = (0x00)
MACON3_PADCFG_60_BYTES      = (0x20)
MACON3_PADCFG_64_BYTES      = (0x60)
MACON3_PADCFG_AUTO          = (0xA0)
MACON3_TXCRCEN              = (0x10)
MACON3_PHDRLEN              = (0x08)
MACON3_HFRMEN               = (0x04)
MACON3_FRMLNEN              = (0x02)
MACON3_FULDPX               = (0x01)

# MAC Control 4 register
MACON4_DEFER                = (0x40)
MACON4_BPEN                 = (0x20)
MACON4_NOBKOFF              = (0x10)
MACON4_LONGPRE              = (0x02)
MACON4_PUREPRE              = (0x01)

# Back-to-Back Inter-Packet Gap register
MABBIPG_DEFAULT_HD          = (0x12)
MABBIPG_DEFAULT_FD          = (0x15)

# Non-Back-to-Back Inter-Packet Gap Low Byte register
MAIPGL_DEFAULT              = (0x12)

# Non-Back-to-Back Inter-Packet Gap High Byte register
MAIPGH_DEFAULT              = (0x0C)

# Retransmission Maximum register
MACLCON1_RETMAX             = (0x0F)

# Collision Window register
MACLCON2_COLWIN             = (0x3F)
MACLCON2_COLWIN_DEFAULT     = (0x37)

# MAC-PHY Support register
MAPHSUP_RSTINTFC            = (0x80)
MAPHSUP_R4                  = (0x10)
MAPHSUP_RSTRMII             = (0x08)
MAPHSUP_R0                  = (0x01)

# MII Control register
MICON_RSTMII                = (0x80)

# MII Command register
MICMD_MIISCAN               = (0x02)
MICMD_MIIRD                 = (0x01)

# MII Register Address register
MIREGADR_VAL                = (0x1F)

# Self-Test Control register
EBSTCON_PSV                 = (0xE0)
EBSTCON_PSEL                = (0x10)
EBSTCON_TMSEL               = (0x0C)
EBSTCON_TMSEL_RANDOM        = (0x00)
EBSTCON_TMSEL_ADDR          = (0x04)
EBSTCON_TMSEL_PATTERN_SHIFT = (0x08)
EBSTCON_TMSEL_RACE_MODE     = (0x0C)
EBSTCON_TME                 = (0x02)
EBSTCON_BISTST              = (0x01)

# MII Status register
MISTAT_R3                   = (0x08)
MISTAT_NVALID               = (0x04)
MISTAT_SCAN                 = (0x02)
MISTAT_BUSY                 = (0x01)

# Ethernet Revision ID register
EREVID_REV                  = (0x1F)
EREVID_REV_B1               = (0x02)
EREVID_REV_B4               = (0x04)
EREVID_REV_B5               = (0x05)
EREVID_REV_B7               = (0x06)

# Clock Output Control register
ECOCON_COCON                = (0x07)
ECOCON_COCON_DISABLED       = (0x00)
ECOCON_COCON_DIV1           = (0x01)
ECOCON_COCON_DIV2           = (0x02)
ECOCON_COCON_DIV3           = (0x03)
ECOCON_COCON_DIV4           = (0x04)
ECOCON_COCON_DIV8           = (0x05)

# Ethernet Flow Control register
EFLOCON_FULDPXS             = (0x04)
EFLOCON_FCEN                = (0x03)
EFLOCON_FCEN_OFF            = (0x00)
EFLOCON_FCEN_ON_HD          = (0x01)
EFLOCON_FCEN_ON_FD          = (0x02)
EFLOCON_FCEN_SEND_PAUSE     = (0x03)

# PHY Control 1 register
PHCON1_PRST                 = (0x8000)
PHCON1_PLOOPBK              = (0x4000)
PHCON1_PPWRSV               = (0x0800)
PHCON1_PDPXMD               = (0x0100)

# Physical Layer Status 1 register
PHSTAT1_PFDPX               = (0x1000)
PHSTAT1_PHDPX               = (0x0800)
PHSTAT1_LLSTAT              = (0x0004)
PHSTAT1_JBRSTAT             = (0x0002)

# PHY Identifier 1 register
PHID1_PIDH                  = (0xFFFF)
PHID1_PIDH_DEFAULT          = (0x0083)

# PHY Identifier 2 register
PHID2_PIDL                  = (0xFC00)
PHID2_PIDL_DEFAULT          = (0x1400)
PHID2_PPN                   = (0x03F0)
PHID2_PPN_DEFAULT           = (0x0000)
PHID2_PREV                  = (0x000F)

# PHY Control 2 register
PHCON2_FRCLNK               = (0x4000)
PHCON2_TXDIS                = (0x2000)
PHCON2_JABBER               = (0x0400)
PHCON2_HDLDIS               = (0x0100)

# Physical Layer Status 2 register
PHSTAT2_TXSTAT              = (0x2000)
PHSTAT2_RXSTAT              = (0x1000)
PHSTAT2_COLSTAT             = (0x0800)
PHSTAT2_LSTAT               = (0x0400)
PHSTAT2_DPXSTAT             = (0x0200)
PHSTAT2_PLRITY              = (0x0010)

# PHY Interrupt Enable register
PHIE_PLNKIE                 = (0x0010)
PHIE_PGEIE                  = (0x0002)

# PHY Interrupt Request register
PHIR_PLNKIF                 = (0x0010)
PHIR_PGIF                   = (0x0004)

# PHY Module LED Control register
PHLCON_LACFG                = (0x0F00)
PHLCON_LACFG_TX             = (0x0100)
PHLCON_LACFG_RX             = (0x0200)
PHLCON_LACFG_COL            = (0x0300)
PHLCON_LACFG_LINK           = (0x0400)
PHLCON_LACFG_DUPLEX         = (0x0500)
PHLCON_LACFG_TX_RX          = (0x0700)
PHLCON_LACFG_ON             = (0x0800)
PHLCON_LACFG_OFF            = (0x0900)
PHLCON_LACFG_BLINK_FAST     = (0x0A00)
PHLCON_LACFG_BLINK_SLOW     = (0x0B00)
PHLCON_LACFG_LINK_RX        = (0x0C00)
PHLCON_LACFG_LINK_TX_RX     = (0x0D00)
PHLCON_LACFG_DUPLEX_COL     = (0x0E00)
PHLCON_LBCFG                = (0x00F0)
PHLCON_LBCFG_TX             = (0x0010)
PHLCON_LBCFG_RX             = (0x0020)
PHLCON_LBCFG_COL            = (0x0030)
PHLCON_LBCFG_LINK           = (0x0040)
PHLCON_LBCFG_DUPLEX         = (0x0050)
PHLCON_LBCFG_TX_RX          = (0x0070)
PHLCON_LBCFG_ON             = (0x0080)
PHLCON_LBCFG_OFF            = (0x0090)
PHLCON_LBCFG_BLINK_FAST     = (0x00A0)
PHLCON_LBCFG_BLINK_SLOW     = (0x00B0)
PHLCON_LBCFG_LINK_RX        = (0x00C0)
PHLCON_LBCFG_LINK_TX_RX     = (0x00D0)
PHLCON_LBCFG_DUPLEX_COL     = (0x00E0)
PHLCON_LFRQ                 = (0x000C)
PHLCON_LFRQ_40_MS           = (0x0000)
PHLCON_LFRQ_73_MS           = (0x0004)
PHLCON_LFRQ_139_MS          = (0x0008)
PHLCON_STRCH                = (0x0002)

# Per-packet control byte
TX_CTRL_PHUGEEN             = (0x08)
TX_CTRL_PPADEN              = (0x04)
TX_CTRL_PCRCEN              = (0x02)
TX_CTRL_POVERRIDE           = (0x01)

# Receive status vector
ENC28J60_RSV_VLAN_TYPE               = (0x4000)
ENC28J60_RSV_UNKNOWN_OPCODE          = (0x2000)
ENC28J60_RSV_PAUSE_CONTROL_FRAME     = (0x1000)
ENC28J60_RSV_CONTROL_FRAME           = (0x0800)
ENC28J60_RSV_DRIBBLE_NIBBLE          = (0x0400)
ENC28J60_RSV_BROADCAST_PACKET        = (0x0200)
ENC28J60_RSV_MULTICAST_PACKET        = (0x0100)
ENC28J60_RSV_RECEIVED_OK             = (0x0080)
ENC28J60_RSV_LENGTH_OUT_OF_RANGE     = (0x0040)
ENC28J60_RSV_LENGTH_CHECK_ERROR      = (0x0020)
ENC28J60_RSV_CRC_ERROR               = (0x0010)
ENC28J60_RSV_CARRIER_EVENT           = (0x0004)
ENC28J60_RSV_DROP_EVENT              = (0x0001)


def gen_Macaddress():
    '''                          Network cable Output sequence
    Mac Address <47:40> = a[0]   a[0] bit 40 -> 41 -> 42 -> 43 -> 44 -> 45 -> 46 -> 47
                                 a[0] bit 40 = 0: Unicast address 1: Multicast address
                                 a[0] bit 41 = 0: Global address  1: Local address
                                 a[0] bit 42-47, a[1],a[2],a[3],a[4],a[5] = MAC address
    Mac Address <39:32> = a[1]   a[1] bit 32 -> 33 -> 34 -> 35 -> 36 -> 37 -> 38 -> 39
    Mac Address <31:24> = a[2]   a[2] bit 24 -> 25 -> 26 -> 27 -> 28 -> 29 -> 30 -> 31
    Mac Address <23:16> = a[3]   a[3] bit 16 -> 17 -> 18 -> 19 -> 20 -> 21 -> 22 -> 23
    Mac Address <15:8>  = a[4]   a[4] bit 8 -> 9 -> 10 -> 11 -> 12 -> 13 -> 14 -> 15
    Mac Address <7:0>   = a[5]   a[5] bit 0 -> 1 -> 2 -> 3 -> 4 -> 5 -> 6 -> 7
    '''
    import uuid
    a = uuid.uuid4().bytes[-6:]
    return bytes([(a[0] & 0xfe)| 0x02]) + a[1:] # set local assignment bit (IEEE802)
                                                # unicast mac address

class ENC28J60:
    def __init__(self, pin_int, pin_reset, maca= None, abooFulldpx=True, abooMulticastRx=False):
        self.aintBnk = 0xFFFF
        self.abooFulldpx = abooFulldpx
        self.abooMulticastRx = abooMulticastRx
        self.abooInt = False
        self.aintCntInt = 0
        # MAC address
        if maca:
            self.abytMacAddr = maca
        else:
            self.abytMacAddr = gen_Macaddress()
        self.aintRevID = None
        self.aintIntPin = pin_int
        self.aintResetPin = pin_reset
        # SPI init
        self.spi = spidev.SpiDev()
        self.spi.open(0, 0)
        self.spi.max_speed_hz = 12000000
        self.spi.mode = 0

        # Interrupt Pin
        GPIO.setmode(GPIO.BCM)
        GPIO.setup(self.aintIntPin, GPIO.IN, GPIO.PUD_UP)
        # Rest Pin
        GPIO.setup(self.aintResetPin, GPIO.OUT)
        GPIO.output(self.aintResetPin, 1)

    def intCallback(cls,interruptPin):
        print("Interrupt")
        cls.aintCntInt += 1
        len, data = cls.ReceivePacket(200)
        if len > 0:
            print("Count:", cls.aintCntInt, " Recv:",len,",Data=",binascii.b2a_hex(data,b' '))

    def enableInt(self,callback):
        if self.abooInt == False:
            GPIO.add_event_detect(self.aintIntPin, GPIO.FALLING, callback=callback)
            self.abooInt = True

    def disableInt(self):
        if self.abooInt == True:
            GPIO.remove_event_detect(self.aintIntPin)
            self.abooInt = False

    def spiReset(self):
        ''' ENC28J60 Soft Reset '''
        GPIO.output(self.aintResetPin, 0)
        time.sleep(0.02)
        GPIO.output(self.aintResetPin, 1)
        time.sleep(1)
        self.spi.xfer2([ENC28J60_CMD_SRC])
        time.sleep(0.2)

    def spiSelectBank(self, reg):
        ''' ENC28J60 select bank '''
        bank = reg & REG_BANK_MASK

        # Rewrite the bank number only if a change is detected
        if (bank == self.aintBnk):
            return
        if bank & 0x100:
            self.spi.xfer2([(ENC28J60_ECON1|ENC28J60_CMD_BFS)&0xFF, 1])
        else:
            self.spi.xfer2([(ENC28J60_CMD_BFC|ENC28J60_ECON1)&0xFF, 1])
        if bank & 0x200:
            self.spi.xfer2([(ENC28J60_CMD_BFS|ENC28J60_ECON1)&0xFF, 2])
        else:
            self.spi.xfer2([(ENC28J60_CMD_BFC|ENC28J60_ECON1)&0xFF, 2])

        self.aintBnk = bank

    def spiReadReg(self, reg):
        ''' Read ENC28J60 Register '''
        self.spiSelectBank(reg)
        if (reg & REG_TYPE_MASK) != ETH_REG_TYPE:
            ret = self.spi.xfer2([(reg|ENC28J60_CMD_RCR)&0xFF, 0x00, 0x00])
            data = ret[2]
        else:
            ret = self.spi.xfer2([(reg|ENC28J60_CMD_RCR)&0xFF, 0x00])
            data = ret[1]

        return data

    def spiWriteReg(self, reg, data):
        ''' Write ENC28J60 Register '''
        self.spiSelectBank(reg)
        self.spi.xfer2([(reg|ENC28J60_CMD_WCR)&0xFF, data])

    def spiClearBitReg(self, reg, mask):
        ''' Clear ENC28J60 Register Bits '''
        self.spi.xfer2([((reg & 0x1f)|ENC28J60_CMD_BFC)&0xFF, mask])

    def spiSetBitReg(self, reg, mask):
        ''' Set ENC28J60 Register Bits '''
        self.spi.xfer2([((reg & 0x1F)|ENC28J60_CMD_BFS)&0xFF, mask])

    def spiWritePhyReg(self, address, data):
        while 0 != (self.spiReadReg(ENC28J60_MISTAT) & (MISTAT_BUSY|MISTAT_NVALID)):
            pass
        # Write register address
        self.spiWriteReg(ENC28J60_MIREGADR, address & 0x1F)

        # Write the lower 8 bits
        self.spiWriteReg(ENC28J60_MIWRL, data & 0xFF)
        # Write the upper 8 bits
        self.spiWriteReg(ENC28J60_MIWRH, data >> 8)

        # Wait until the PHY register has been written
        while 0 != (self.spiReadReg(ENC28J60_MISTAT) & MISTAT_BUSY):
            pass
        return

    def spiReadPhyReg(self, address):
        while 0 != (self.spiReadReg(ENC28J60_MISTAT) & (MISTAT_BUSY|MISTAT_NVALID)):
            pass
        # Write register address
        self.spiWriteReg(ENC28J60_MIREGADR, address & 0x1F)

        # Start read operation
        self.spiWriteReg(ENC28J60_MICMD, MICMD_MIIRD)

        # Wait for the read operation to complete
        while 0 != (self.spiReadReg(ENC28J60_MISTAT) & MISTAT_BUSY):
            pass

        # Clear command register
        self.spiWriteReg(ENC28J60_MICMD, 0)

        # Read the lower 8 bits
        data = self.spiReadReg(ENC28J60_MIRDL)
        # Read the upper 8 bits
        data |= self.spiReadReg(ENC28J60_MIRDH) << 8

        return data

    def getMacAddr(self):
        return self.abytMacAddr

    def enc28j60_dump_regs(self, msg):
        print("..... {:s} .....".format(msg))
        print(". HwRevID: 0x{:02X}".format(self.spiReadReg(ENC28J60_EREVID)))
        print(". Cntrl: ECON1 ECON2 ESTAT  EIR  EIE\n.        0x{:02X}  0x{:02X}  0x{:02X}  0x{:02X}  0x{:02X}".format(self.spiReadReg(ENC28J60_ECON1), self.spiReadReg(ENC28J60_ECON2), self.spiReadReg(ENC28J60_ESTAT),self.spiReadReg(ENC28J60_EIR), self.spiReadReg(ENC28J60_EIE)))
        print(". MAC  : MACON1 MACON3 MACON4\n.        0x{:02X}   0x{:02X}   0x{:02X}".format(self.spiReadReg(ENC28J60_MACON1),self.spiReadReg(ENC28J60_MACON3), self.spiReadReg(ENC28J60_MACON4)))
        print(". Rx   : ERXST  ERXND  ERXWRPT ERXRDPT ERXFCON EPKTCNT MAMXFL\n.        0x{:02X}{:02X} 0x{:02X}{:02X} 0x{:02X}{:02X}  0x{:02X}{:02X}  0x{:02X}    0x{:02X}    0x{:02X}{:02X}".format(self.spiReadReg(ENC28J60_ERXSTH),self.spiReadReg(ENC28J60_ERXSTL),self.spiReadReg(ENC28J60_ERXNDH),self.spiReadReg(ENC28J60_ERXNDL), self.spiReadReg(ENC28J60_ERXWRPTH),self.spiReadReg(ENC28J60_ERXWRPTL),self.spiReadReg(ENC28J60_ERXRDPTH),self.spiReadReg(ENC28J60_ERXRDPTL), self.spiReadReg(ENC28J60_ERXFCON), self.spiReadReg(ENC28J60_EPKTCNT),self.spiReadReg(ENC28J60_MAMXFLH),self.spiReadReg(ENC28J60_MAMXFLL)))
        print(". Tx   : ETXST  ETXND  MACLCON1 MACLCON2 MAPHSUP\n.        0x{:02X}{:02X} 0x{:02X}{:02X} 0x{:02X}     0x{:02X}     0x{:02X}".format(self.spiReadReg(ENC28J60_ETXSTH),self.spiReadReg(ENC28J60_ETXSTL),self.spiReadReg(ENC28J60_ETXNDH),self.spiReadReg(ENC28J60_ETXNDL), self.spiReadReg(ENC28J60_MACLCON1),self.spiReadReg(ENC28J60_MACLCON2), self.spiReadReg(ENC28J60_MAPHSUP)))
        print("..........")

    def enc28j60_dump_phyregs(self, msg):
        print("------ {:s} ------".format(msg))
        print("- PHCON1:{:04x}".format(self.spiReadPhyReg(ENC28J60_PHCON1)))
        print("- PHCON2:{:04x}".format(self.spiReadPhyReg(ENC28J60_PHCON2)))
        print("- PHSTAT1:{:04x}".format(self.spiReadPhyReg(ENC28J60_PHSTAT1)))
        print("- PHSTAT2:{:04x}".format(self.spiReadPhyReg(ENC28J60_PHSTAT2)))
        print("- PHID1:{:04x}".format(self.spiReadPhyReg(ENC28J60_PHID1)))
        print("- PHID2:{:04x}".format(self.spiReadPhyReg(ENC28J60_PHID2)))
        print("------------")


    def init(self):
        self.spiReset() # reset enc28j60
        self.aintRevID = self.spiReadReg(ENC28J60_EREVID)  # read enc28j60 EREVID
        self.nextPacket = ENC28J60_RX_BUFFER_START

        self.spiWriteReg(ENC28J60_ECOCON, ECOCON_COCON_DISABLED)  # Disable clock out pin

        self.spiWriteReg(ENC28J60_MAADR5, self.abytMacAddr[0])  # Set Mac Address <47:40> 
        self.spiWriteReg(ENC28J60_MAADR4, self.abytMacAddr[1])  # Set Mac Address <39:32>
        self.spiWriteReg(ENC28J60_MAADR3, self.abytMacAddr[2])  # Set Mac Address <31:24>
        self.spiWriteReg(ENC28J60_MAADR2, self.abytMacAddr[3])  # Set Mac Address <23:16>
        self.spiWriteReg(ENC28J60_MAADR1, self.abytMacAddr[4])  # Set Mac Address <15:8>
        self.spiWriteReg(ENC28J60_MAADR0, self.abytMacAddr[5])  # Set Mac Address <7:0>
        # Set recv buffer location
        self.spiWriteReg(ENC28J60_ERXSTL, ENC28J60_RX_BUFFER_START & 0xFF)
        self.spiWriteReg(ENC28J60_ERXSTH, ENC28J60_RX_BUFFER_START >> 8)
        self.spiWriteReg(ENC28J60_ERXNDL, ENC28J60_RX_BUFFER_STOP & 0xFF)
        self.spiWriteReg(ENC28J60_ERXNDH, ENC28J60_RX_BUFFER_STOP >> 8)
        # Set send buffer location
        self.spiWriteReg(ENC28J60_ETXSTL, ENC28J60_TX_BUFFER_START & 0xFF)
        self.spiWriteReg(ENC28J60_ETXSTH, ENC28J60_TX_BUFFER_START >> 8)
        self.spiWriteReg(ENC28J60_ETXNDL, ENC28J60_TX_BUFFER_START & 0xFF)
        self.spiWriteReg(ENC28J60_ETXNDH, ENC28J60_TX_BUFFER_START >> 8)
        # The ERXRDPT register 
        self.spiWriteReg(ENC28J60_ERXRDPTL, ENC28J60_RX_BUFFER_STOP & 0xFF)
        self.spiWriteReg(ENC28J60_ERXRDPTH, ENC28J60_RX_BUFFER_STOP >> 8)

        # Configure the receive filters
        if self.abooMulticastRx:
            self.spiWriteReg(ENC28J60_ERXFCON, ERXFCON_UCEN | ERXFCON_CRCEN | ERXFCON_BCEN | ERXFCON_MCEN)
        else:
            self.spiWriteReg(ENC28J60_ERXFCON, ERXFCON_UCEN | ERXFCON_CRCEN | ERXFCON_BCEN )

        # MAC out of reset
        self.spiWriteReg(ENC28J60_MACON2, 0x00)

        # Enable the MAC to recv
        self.spiWriteReg(ENC28J60_MACON1, MACON1_TXPAUS | MACON1_RXPAUS | MACON1_MARXEN)

        # Enable automatic padding in half-duplex or full-duplex mode
        if self.abooFulldpx:
            self.spiWriteReg(ENC28J60_MACON3, MACON3_PADCFG_AUTO | MACON3_TXCRCEN | MACON3_FRMLNEN | MACON3_FULDPX)
        else:
            self.spiWriteReg(ENC28J60_MACON3, MACON3_PADCFG_AUTO | MACON3_TXCRCEN | MACON3_FRMLNEN)

        # MACON4
        self.spiWriteReg(ENC28J60_MACON4, MACON4_DEFER)

        # Maximum frame received or transmitted
        self.spiWriteReg(ENC28J60_MAMXFLL, ENC28J60_ETH_RX_BUFFER_SIZE & 0xFF)
        self.spiWriteReg(ENC28J60_MAMXFLH, ENC28J60_ETH_RX_BUFFER_SIZE >> 8)

        # back-to-back inter-packet gap register
        if self.abooFulldpx:
            self.spiWriteReg(ENC28J60_MABBIPG, MABBIPG_DEFAULT_FD)
        else:
            self.spiWriteReg(ENC28J60_MABBIPG, MABBIPG_DEFAULT_HD)

        # non-back-to-back inter-packet gap register
        self.spiWriteReg(ENC28J60_MAIPGL, MAIPGL_DEFAULT)
        self.spiWriteReg(ENC28J60_MAIPGH, MAIPGH_DEFAULT)

        # Collision window register
        self.spiWriteReg(ENC28J60_MACLCON2, MACLCON2_COLWIN_DEFAULT)
        #self.spiWritePhyReg(ENC28J60_PHCON1, PHCON1_PRST)
        # Set the PHY to the proper duplex mode
        if self.abooFulldpx:
            self.spiWritePhyReg(ENC28J60_PHCON1, PHCON1_PDPXMD)
        else:
            self.spiWritePhyReg(ENC28J60_PHCON1, 0x0000)

        # Disable half-duplex loopback in PHY
        self.spiWritePhyReg(ENC28J60_PHCON2, PHCON2_HDLDIS)

        # LEDA displays link status and LEDB displays TX/RX activity
        #self.spiWritePhyReg(ENC28J60_PHLCON, PHLCON_LACFG_LINK | PHLCON_LBCFG_LINK | PHLCON_LFRQ_40_MS | PHLCON_STRCH)

        # Clear PHY interrupts flags
        self.spiReadPhyReg(ENC28J60_PHIR)

        # Clear EIR
        self.spiClearBitReg(ENC28J60_EIR, EIR_TXIF | EIR_TXERIF )

        # Set EIE
        self.spiWriteReg(ENC28J60_EIE, EIE_INTIE | EIE_PKTIE |EIE_LINKIE)
        # | ENC28J60_EIE_TXIE | ENC28J60_EIE_TXERIE)

        # PHY PHIE
        self.spiWritePhyReg(ENC28J60_PHIE, PHIE_PLNKIE | PHIE_PGEIE)

        # Set RXEN
        self.spiSetBitReg(ENC28J60_ECON1, ECON1_RXEN)
        self.enc28j60_dump_regs("INIT() exit")

    def GetRevId(self):
        return self.aintRevID

    def spiReadBuffer(self, size):
        rb = bytearray(size+1)
        rb[0] = ENC28J60_CMD_RBM
        # Read opcode
        data = self.spi.xfer2(rb)[1:]

        # Read data from SRAM buffer
        return data

    def spiWriteBuffer(self, chunks):
        n = 0
        for i in chunks:
            n += 1
        bf = bytearray(n+2)
        # Write opcode
        bf = bytes([ENC28J60_CMD_WBM, 0])
        bf += chunks 
        #self.spi.xfer2([ENC28J60_CMD_WBM])
        #self.spi.xfer2([0x00])             # Set control packet byte
        # Loop through data chunks
        self.spi.xfer2(bf)

    def spiIsLinkUp(self):
        return 0 != (self.spiReadPhyReg(ENC28J60_PHSTAT2) & PHSTAT2_LSTAT)

    def spiIsLinkStateChanged(self):
        # Read interrupt status register
        status = self.spiReadReg(ENC28J60_EIR)

        # Check whether the link state has changed
        if 0 == (status & EIR_LINKIF):
            return False

        # Clear PHY interrupts flags
        self.spiReadPhyReg(ENC28J60_PHIR)
        self.spiClearBitReg(ENC28J60_EIR, EIR_LINKIF)
        return True

    def GetRxPacketCnt(self):
        return self.spiReadReg(ENC28J60_EPKTCNT)

    def SendPacket(self, chunks):
        length = 0
        for i in chunks:
            length += 1
        #length = len(chunks)

        # Check the frame length
        if length > ENC28J60_ETH_TX_BUFFER_SIZE:
            return ENC28J60_ETH_TX_ERR_MSGSIZE

        # Make sure the link is up before transmitting the frame
        if False == self.spiIsLinkUp():
            return ENC28J60_ETH_TX_ERR_LINKDOWN
        # If TX Error to reset the transmit logic
        if (self.spiReadReg(ENC28J60_ECON1) & 0x08) != 0:
            pass
        #self.spiSetBitReg(ENC28J60_ECON1, ECON1_TXRST)
        #self.spiClearBitReg(ENC28J60_ECON1, ECON1_TXRST)
        # Interrupt flags should be cleared after the reset is completed
        #self.spiClearBitReg(ENC28J60_EIR, EIR_TXIF | EIR_TXERIF)

        # Point to start of transmit buffer
        self.spiWriteReg(ENC28J60_ETXSTL, ENC28J60_TX_BUFFER_START & 0xFF)
        self.spiWriteReg(ENC28J60_ETXSTH, ENC28J60_TX_BUFFER_START >> 8)
        self.spiWriteReg(ENC28J60_EWRPTL, ENC28J60_TX_BUFFER_START & 0xFF)
        self.spiWriteReg(ENC28J60_EWRPTH, ENC28J60_TX_BUFFER_START >> 8)
        # ETXND should point to the last byte in the data payload
        self.spiWriteReg(ENC28J60_ETXNDL, (ENC28J60_TX_BUFFER_START + length) & 0xFF)
        self.spiWriteReg(ENC28J60_ETXNDH, (ENC28J60_TX_BUFFER_START + length) >> 8)

        # Copy the data to the transmit buffer
        self.spiWriteBuffer(chunks)
        self.spiClearBitReg(ENC28J60_EIR, 0x08)

        # Start transmission
        self.spiSetBitReg(ENC28J60_ECON1, ECON1_TXRTS)
        if (self.spiReadReg(ENC28J60_EIR) & EIR_TXERIF) != 0:
            # If TX Error to reset the transmit logic
            self.spiSetBitReg(ENC28J60_ECON1, ECON1_TXRST)
            self.spiClearBitReg(ENC28J60_ECON1, ECON1_TXRTS)
            return  ENC28J60_ETH_TX_ERR_SEND            
        return length

    def ReceivePacket(self, size):
        if 0 == self.GetRxPacketCnt():
            return [0, bytes(1)]
        
        # Point to the start of the received packet
        self.spiWriteReg(ENC28J60_ERDPTL, self.nextPacket & 0xFF)
        self.spiWriteReg(ENC28J60_ERDPTH, self.nextPacket >> 8)

        # 6-byte header
        r6b = self.spi.xfer2([ENC28J60_CMD_RBM,0,0,0,0,0,0])[1:]

        #header6b = self.spiReadBuffer(6)
        # The first two bytes are the address of the next packet
        self.nextPacket = r6b[0] | (r6b[1]<<8)

        # Get the length of the received packet
        length = r6b[2] | (r6b[3]<<8)

        # Get the receive status vector (RSV)
        status = r6b[4] | (r6b[5]<<8)

        # Make sure no error occurred
        if 0 != (status & ENC28J60_RSV_RECEIVED_OK):
            # Limit the number of data to read
            length = min(length, ENC28J60_ETH_RX_BUFFER_SIZE)
            length = min(length, size)

            # Read the Ethernet frame
            length -= 4
            data = self.spiReadBuffer(length)
        else:
            # The received packet contains an error
            length = -1
            data = bytes()

        # Advance the ERXRDPT pointer, taking care to wrap back at the end of the received memory buffer
        if ENC28J60_RX_BUFFER_START == self.nextPacket:
            self.spiWriteReg(ENC28J60_ERXRDPTL, ENC28J60_RX_BUFFER_STOP & 0xFF)
            self.spiWriteReg(ENC28J60_ERXRDPTH, ENC28J60_RX_BUFFER_STOP << 8)
        else:
            self.spiWriteReg(ENC28J60_ERXRDPTL, (self.nextPacket) & 0xFF)
            self.spiWriteReg(ENC28J60_ERXRDPTH, (self.nextPacket) << 8)

        # Decrement the packet counter
        self.spiSetBitReg(ENC28J60_ECON2, ECON2_PKTDEC)
        return [length, bytes(data)]

    def deInit(self):
        self.spi.close()
        GPIO.cleanup()

if __name__ == '__main__':
    enc = ENC28J60(25, 24, maca=b'\x3e\x3f\x3f\x02\x11\x22')
    print("My MAC Adrress:", binascii.b2a_hex(enc.abytMacAddr,b':'))
    enc.init()
    for i in range(3):
        if enc.spiIsLinkUp():
            print("Link UP")
        else:
            print("Link DOWN")
    if enc.spiIsLinkUp():
        pass
    else:
        enc.spiWritePhyReg(ENC28J60_PHLCON, 0xaa2)
        t = 0
        while 1:
            time.sleep(1)
            t += 1
            if t >= 20:
                break
        enc.disableInt()
        enc.deInit()
        quit()
    enc.enc28j60_dump_phyregs("PHY Register")
    msg = b'\xff\xff\xff\xff\xff\xff' + enc.abytMacAddr + b'\x08\x06\x00\x01\x08\x00\x06\x04\x00\x01' + enc.abytMacAddr + b'\xc0\xa8\x01\x27\x00\x00\x00\x00\x00\x00\xc0\xa8\x01\x01'
    enc.enableInt(enc.intCallback)
    enc.SendPacket(msg)
    n1 = 0
    try:
        while True:
            if enc.abooInt == False:
                len, data = enc.ReceivePacket(200)
                if len > 0:
                    print("Recv:",len,",Data=",binascii.b2a_hex(data,b' '))
            time.sleep(1)
            n1 += 1
            print(n1 , " Sec ..")
            if n1 >= 15:
                break
        enc.disableInt()
        enc.deInit()

    except KeyboardInterrupt:
        print('Quit')
        enc.disableInt()
        enc.deInit()


'''
$ python enc28j60.py 
My MAC Adrress: b'3e:3f:3f:02:11:22'
..... INIT() exit .....
. HwRevID: 0x04
. Cntrl: ECON1 ECON2 ESTAT  EIR  EIE
.        0x04  0x80  0x01  0x00  0xD0
. MAC  : MACON1 MACON3 MACON4
.        0x0D   0xB3   0x40
. Rx   : ERXST  ERXND  ERXWRPT ERXRDPT ERXFCON EPKTCNT MAMXFL
.        0x0000 0x17FF 0x0000  0x17FF  0xA1    0x00    0x05EE
. Tx   : ETXST  ETXND  MACLCON1 MACLCON2 MAPHSUP
.        0x1800 0x1800 0x0F     0x37     0x10
..........
Link UP
Link UP
Link UP
------ PHY Register ------
- PHCON1:0100
- PHCON2:0100
- PHSTAT1:1804
- PHSTAT2:0600
- PHID1:0083
- PHID2:1400
------------
1  Sec ..
Recv: 60 ,Data= b'3e 3f 3f 02 11 22 XX XX XX X6 ed d5 08 06 00 01 08 00 06 04 00 02 XX XX XX X6 ed d5 c0 a8 01 01 3e 3f 3f 02 11 22 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
Recv: 60 ,Data= b'3e 3f 3f 02 11 22 XX XX XX X6 ed d5 08 06 00 01 08 00 06 04 00 01 XX XX XX X6 ed d5 c0 a8 01 01 3e 3f 3f 02 11 22 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
2  Sec ..
Recv: 60 ,Data= b'3e 3f 3f 02 11 22 XX XX XX X6 ed d5 08 06 00 01 08 00 06 04 00 01 XX XX XX X6 ed d5 c0 a8 01 01 3e 3f 3f 02 11 22 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
3  Sec ..
Recv: 60 ,Data= b'3e 3f 3f 02 11 22 XX XX XX X6 ed d5 08 06 00 01 08 00 06 04 00 01 XX XX XX X6 ed d5 c0 a8 01 01 3e 3f 3f 02 11 22 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
4  Sec ..
Recv: 60 ,Data= b'ff ff ff ff ff ff XX XX XX X6 ed d5 08 06 00 01 08 00 06 04 00 01 XX XX XX X6 ed d5 c0 a8 01 01 00 00 00 00 00 00 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
5  Sec ..
Recv: 60 ,Data= b'ff ff ff ff ff ff XX XX XX X6 ed d5 08 06 00 01 08 00 06 04 00 01 XX XX XX X6 ed d5 c0 a8 01 01 00 00 00 00 00 00 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
6  Sec ..
Recv: 60 ,Data= b'ff ff ff ff ff ff XX XX XX X6 ed d5 08 06 00 01 08 00 06 04 00 01 XX XX XX X6 ed d5 c0 a8 01 01 00 00 00 00 00 00 c0 a8 01 27 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
7  Sec ..
8  Sec ..
9  Sec ..
10  Sec ..
11  Sec ..
12  Sec ..
13  Sec ..
14  Sec ..
15  Sec ..
'''